#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2022 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import heat as h
import heat.utils as u
import tbox
import tbox.gmsh as gmsh
from fwk.testing import *

def main(**d):

    # default params
    msh = gmsh.MeshLoader("mat2.geo",__file__).execute()
    gmshWriter = tbox.GmshExport(msh)
    pbl = h.Problem(msh)

    fa=tbox.PwLf()
    fa.add(0, 100)
    fa.add(150, 100)
    fa.add(300, 100)

    fb=tbox.PwLf()
    fb.add(0,   10)
    fb.add(130, 10)
    fb.add(170, 1000)
    fb.add(300, 1000)

    fz = tbox.PwLf()
    fz.add(0,0)

    pbl.add(h.Medium(msh, "Matrix", tbox.Fct2PwLf(fa,fa,fz)))
    pbl.add(h.Medium(msh, "Inclusion", tbox.Fct2PwLf(fb,fb,fz)))
    
    pbl.add(h.Dirichlet(msh, "Left", 0.))
    pbl.add(h.Dirichlet(msh, "Right", 300.))
    
    # solve problem
    solver = u.solve(pbl, gmshWriter)
    
    # extract some results
    x1,y1,T1 = u.getTemp(solver,"Top", sort='x')
    
    u.simpleplot(x1, T1, xlabel='x [mm]', title='Temperature along x')
       
    tests = CTests()
    tests.add(CTest('mean(Ttop)', sum(T1) / len(T1), 169, 1., forceabs=True))
    tests.add(CTest('max(Ttop)', max(T1), 300))
    tests.add(CTest('min(Ttop)', min(T1), 0., refval=300))
    tests.run()
    
if __name__ == "__main__":
    main()
